from __future__ import with_statement 
from vcCommand import *
from vcHelpers.Selection import *
from vcHelpers.Application import *
import math, vcVector, vcMatrix
import os.path
import datetime
from PIL import Image

app = getApplication()

def sanitizefilename(fname):
  suspiciouscharacters = ["#:;!%&?^* \t"]
  cname = ""
  for c in fname:
    if c in suspiciouscharacters:
      cname += "_"
    else:
      cname += c
  return cname

def first_state():
  global button, ignoretransparentsproperty, eachnodeinownfile
  ignoretransparentsproperty = addProperty(getCommand(),VC_BOOLEAN,'IgnoreTransparentItems',True,False,False,True,None,'')
  eachnodeinownfile = addProperty(getCommand(),VC_BOOLEAN,'EachNodeInOwnFile',True,False,False,True,None,'')
  button = createProperty(VC_BUTTON, 'Save')
  button.OnChanged = doTheOBJ
  executeInActionPanel()


def doTheOBJ(arg):
  save = app.findCommand("dialogSaveFile")
  uri = ""
  ok = True
  filter = "OBJ model file (*.obj)|*.obj"
  save.execute(uri,ok,filter,'Choose File to save model to')
  if not save.Param_2:
    print "No file selected, aborting command."
    return
  uri = save.Param_1
  filename = uri[8:len(uri)] 
  basename, extension = os.path.splitext(os.path.basename(filename))
  dirname = os.path.dirname(filename)
  mtlfilename = dirname +'\\'+ basename + ".mtl"

  if not eachnodeinownfile.Value:
    try:
      out = open(sanitizefilename(filename),"w")
    except:
      print "Cannot open file for saving <%s>" % (filename)
      return
    out.write("# Visual Components OBJ Exporter v0.1b - (c)2013 Visual Components\n")
    out.write("# File Created: %s\n\n" % datetime.datetime.now().isoformat())
    out.write("mtllib %s\n\n" % (basename+".mtl"))

  dirM = vcMatrix.new()
  materials = {}
  ii = 1
  for n in getAllComponentsNodes():
    if eachnodeinownfile.Value:
      hasgeometry = False
      for gs in n.Geometry.GeometrySets:
        if gs.Type == VC_TRIANGLESET:
          if gs.PointCount > 0:
            hasgeometry = True
            break
      if hasgeometry:
        try:
          out = open(sanitizefilename(dirname +'\\'+ basename+"_"+n.Component.Name+"_"+n.Name+".obj"),"w")
        except:
          print "Cannot open file for saving <%s>" % (filename)
          return
        out.write("# Visual Components OBJ Exporter v0.1b - (c)2013 Visual Components\n")
        out.write("# File Created: %s\n\n" % datetime.datetime.now().isoformat())
        out.write("mtllib %s\n\n" % (basename+".mtl"))
        ii = 1
    mp = n.WorldPositionMatrix
    mo = vcMatrix.new(mp)
    mo.P = vcVector.new()
    for gs in n.Geometry.GeometrySets:
      if gs.Type == VC_TRIANGLESET:
        if gs.Material:
          if ignoretransparentsproperty.Value:
            if gs.Material.OpacityType != VC_MATERIAL_TRANSPARENCY_NONE:
              continue
            if gs.Material.Texture and gs.Material.Texture.Format == VC_BITMAP_RGBA:
              continue
          materials[gs.Material.Name] = gs.Material
        else:
          materials["orange"] = app.findMaterial("orange")
        out.write("#\n# object %s\n#\n\n" % gs.Feature.Name)
        for vec in zip(gs.PositionTable[::3],gs.PositionTable[1::3],gs.PositionTable[2::3]):
          v = mp*vcVector.new(vec[0],vec[1],vec[2])
          out.write("v %.4f %.4f %.4f\n" % (v.X,v.Y,v.Z))
        out.write("# %d vertices\n\n" % gs.PointCount )
        for vec in gs.getPointNormals([i for i in range(gs.PointCount)]):
          v = mo*vec
          out.write("vn %.4f %.4f %.4f\n" % (v.X,v.Y,v.Z))
        out.write("# %d vertex normals\n\n" % gs.PointCount )
        if len(gs.TextureCoordinateTable) > 0:
          for vec in zip(gs.TextureCoordinateTable[::2],gs.TextureCoordinateTable[1::2]):
            out.write("vt %.4f %.4f 0.0000\n" % vec)
        else:
          for i in range(gs.PointCount):
            out.write("vt 0.0000 0.0000 0.0000\n")
        out.write("# %d texture coords\n\n" % gs.PointCount )
        tt = gs.TriangleTable
        out.write("g %s\n" % gs.Feature.Name)
        if gs.Material:
          out.write("usemtl %s\n" % gs.Material.Name)
        else:
          out.write("usemtl orange\n")
        for v in zip(tt[::3],tt[1::3],tt[2::3]):
          out.write("f %d/%d/%d %d/%d/%d %d/%d/%d\n" % (v[0]+ii,v[0]+ii,v[0]+ii,v[1]+ii,v[1]+ii,v[1]+ii,v[2]+ii,v[2]+ii,v[2]+ii))
        out.write("# %d polygons\n\n" % gs.TriangleCount )
        ii += gs.PointCount
    if eachnodeinownfile.Value:
      out.close()
  if not eachnodeinownfile.Value:
    out.close()
  
  try:
    out = open(sanitizefilename(mtlfilename),"w")
  except:
    print "Cannot open material file for saving <%s>" % (mtlfilename)
    return
  out.write("# Visual Components OBJ Exporter v0.1b - (c)2013 Visual Components\n")
  out.write("# File Created: %s\n\n" % datetime.datetime.now().isoformat())

  for mn, mat in materials.iteritems():
    out.write("newmtl %s\n" % mn)
    if mat.OpacityType == VC_MATERIAL_TRANSPARENCY_CONSTANT:
      out.write("\td %.4f\n" % mat.Opacity)
    elif mat.OpacityType == VC_MATERIAL_TRANSPARENCY_NONE:
      out.write("\td 1.0000\n")
    out.write("\tKa %.4f %.4f %.4f\n" % (mat.Ambient.X,mat.Ambient.Y,mat.Ambient.Z))
    out.write("\tKd %.4f %.4f %.4f\n" % (mat.Diffuse.X,mat.Diffuse.Y,mat.Diffuse.Z))
    out.write("\tKs %.4f %.4f %.4f\n" % (mat.Specular.X,mat.Specular.Y,mat.Specular.Z))
    if mat.Texture:
      if mat.Texture.Format == VC_BITMAP_RGBA:
        tname = basename + '_' + mn + ".tga"
      else:
        tname = basename + '_' + mn + ".bmp"
      texturefilename = "file:///" + os.path.dirname(filename) +'\\'+ tname
      app.saveBitmap(mat.Texture, texturefilename)
      #
      # USE PNG INSTEAD
      png = Image.open(texturefilename[8:])
      pngname = os.path.dirname(filename) +'\\'+ basename + '_' + mn + ".png"
      png.save(pngname)
      #os.remove(texturefilename[8:]) # does not remove it, because it is open...
      tname = basename + '_' + mn + ".png"
      #
      out.write("\tmap_Ka %s\n" % tname)
      out.write("\tmap_Kd %s\n" % tname)
    out.write("\n")
  out.close()
  
  
  print "Done"

addState( first_state )