from __future__ import with_statement 
from vcCommand import *
from vcHelpers.Application import *
import math, vcVector, vcMatrix
import os.path
import string

app = getApplication()
matData = None

def stripNonAscii(s):
  ss = ""
  for c in s:
    if ord(c) <128:
      ss += c
  return ss

class ObjGroup:
  def __init__(self,name):
    self.Name = name
    self.MaterialName = ""
    self.Faces = []

class ObjMaterial:
  def __init__(self,name):
    self.Name = name
    self.Opacity = 1.0
    self.Ambient = vcVector.new(0,0,0)
    self.Diffuse = vcVector.new(0,0,0)
    self.Specular = vcVector.new(0,0,0)
    self.Texture = "" 

def loadMAT(filename):
  materials = {}
  material_name = 'OBJ_import_material'
  try:
    with open(filename, "r") as objfile:
      for line in objfile:
        line = stripNonAscii(line.strip())
        if not line or line[0] == '#':
          # skip empty or comment lines
          continue
        vals = line.split() 
        if vals[0] == "newmtl":
          material_name = ' '.join(vals[1:]).strip(',.-#')
          if material_name not in materials:
            materials[material_name] = ObjMaterial(material_name) 
        if vals[0] == "d":
          materials[material_name].Opacity = float(eval(vals[1]))
        if vals[0] == "Ka":
          pass
          materials[material_name].Ambient = vcVector.new(eval(vals[1]),eval(vals[2]),eval(vals[3]))
        if vals[0] == "Kd":
          #materials[material_name].Ambient = vcVector.new(eval(vals[1]),eval(vals[2]),eval(vals[3]))
          materials[material_name].Diffuse = vcVector.new(eval(vals[1]),eval(vals[2]),eval(vals[3]))
          #materials[material_name].Specular = vcVector.new(eval(vals[1]),eval(vals[2]),eval(vals[3]))
        if vals[0] == "Ks":
          pass
          materials[material_name].Specular = vcVector.new(eval(vals[1]),eval(vals[2]),eval(vals[3]))
        if vals[0] == "map_Ka":
          materials[material_name].Texture = vals[1]
        if vals[0] == "map_Kd":
          materials[material_name].Texture = vals[1]
  except:
    pass
  return materials

def loadOBJ(filename,c):
  global matData
  num_lines = sum(1 for line in open(filename))
  currentline = 0
  groups = {}
  group_name = 'OBJ_import_geometry'
  verts = []  
  texts = []
  vindex, tindex = 0, 0
  mtllib = ""
  matData = None
  base_group_name = group_name
  prev_time = app.RealTime
  with open(filename, "r") as objfile:
    for line in objfile:
      currentline += 1
      if currentline % 10000 == 0:
        progressStdMessage(currentline,num_lines)
      line = stripNonAscii(line.strip())
      if not line or line[0] == '#':
        # skip empty or comment lines
        continue
      vals = line.split() 
      if vals[0]=="o":
        # FLUSH EVERYTHITNG HERE
        if buildproperty.Value and len(groups) > 0:
          buildmodel(groups,matData,c)
          vindex += len(verts)
          tindex += len(texts)
          verts, texts = [], []
          groups = {}
          if progressproperty.Value:
            if app.RealTime - prev_time > 10.0:
              app.findCommand("fillView").execute()
              app.render()
              prev_time = app.RealTime
        group_name = ' '.join(vals[1:])
        base_group_name = group_name
        if group_name not in groups:
          groups[group_name] = ObjGroup(group_name) 
      if vals[0] == "g":
        group_name = ' '.join(vals[1:])
        base_group_name = group_name
        if group_name not in groups:
          groups[group_name] = ObjGroup(group_name) 
      if vals[0] == "usemtl":
        matname = ' '.join(vals[1:]).strip(',.-#')
        #if group_name in groups and len(groups[group_name].Faces) == 0: # remove empty groups
        #if len(groups[group_name].Faces) == 0: # remove empty groups
        #  del groups[group_name]
        group_name = base_group_name + '_' + matname
        if group_name not in groups:
          groups[group_name] = ObjGroup(group_name) 
        groups[group_name].MaterialName = matname
      if vals[0] == "mtllib":
        mtllib = ' '.join(vals[1:])
        matData = loadMAT(os.path.join(os.path.dirname(filename),mtllib))
      if vals[0] == "v":
        # vertices
        v = tuple(map(float, vals[1:4]))
        verts.append(v)
      if vals[0] == "vt":
        # vertex texture coordinates
        t = tuple(map(float, vals[1:3])) # we use only 2 coordinates (u,v)
        texts.append(t)
      if vals[0] == "f":
        face = ([],[])
        # FACE
        for f in vals[1:]:  
          w = f.split("/")  
          # FACE VERTEX
          #print face, w, int(w[0])-1-vindex, len(verts)
          if int(w[0]) < 0: #negative indexing 
            # indices are specified relative to the current maximum vertex position (-1 references the last vertex defined)
            face[0].append( verts[int(w[0])] ) # obj indexing starts from 1
          else:
            try:face[0].append( verts[int(w[0])-1-vindex] ) # obj indexing starts from 1
            except:
              print currentline,'<= on line. Error: vertex indexing', len(verts), [int(w[0])-1-vindex]
              return
          #try: face[0].append( verts[int(w[0])-1-vindex] ) # obj indexing starts from 1
          #except: print "ERROR IN ", verts, w[0], vindex
          # FACE TEXTURE COORDINATE
          try:     face[1].append( texts[int(w[1])-1-tindex] )
          except:  face[1].append( (0,0) ) # we use only 2 coordinates (u,v)
        # ADD FACE TO THIS GROUP 
        groups[group_name].Faces.append(face)  #(groups ~ dictionary)
  if len(groups) > 0:
    buildmodel(groups,matData,c)
    vindex += len(verts)
    tindex += len(texts)
    verts, texts = [], []
    groups = {}
    if progressproperty.Value:
      app.findCommand("fillView").execute()
      app.render()
  return groups, matData 

def addFacesToGeometry(geo,faces,matname):
  tset = geo.createGeometrySet(VC_TRIANGLESET)
  position_table = []
  polygon_table = []
  text_coords = []
  for face in faces:
    table_length = len( position_table )/3
    position_table.extend( [token for vec in face[0] for token in vec] )
    text_coords.extend( [token for vec in face[1] for token in vec] )
    polygon_table.append( len(face[0]) )
    polygon_table.extend( [x+table_length for x in range( len(face[0]))] )
  tset.PositionTable = position_table
  tset.TextureCoordinateTable = text_coords
  tset.PolygonTable = polygon_table
  if matData and matname:
    mtl = matData[matname]
    #matname = "OBJ_" + mtl.Name 
    matname = mtl.Name # since we are moving models from 3ds and back, keep naming as is
    mat = app.findMaterial(matname)
    if not mat:
      mat = app.createMaterial(matname)
      mat.Ambient = mtl.Ambient
      mat.Diffuse = mtl.Diffuse
      mat.Specular = mtl.Specular
      mat.Opacity = 1.0#mtl.Opacity
      if mtl.Opacity < 1.0: # transparency 
        mat.OpacityType = VC_MATERIAL_TRANSPARENCY_CONSTANT
      else:
        mat.OpacityType = VC_MATERIAL_TRANSPARENCY_NONE
      if mtl.Texture:
        if mtl.Opacity < 1.0: # transparency map ?
          mat.OpacityType = VC_MATERIAL_TRANSPARENCY_MAP
        print 'TTTt',os.path.join(os.path.dirname(filename),os.path.basename(mtl.Texture))
        try: mat.Texture = app.loadBitmap(os.path.join(os.path.dirname(filename),os.path.basename(mtl.Texture)))
        except: print 'Failed to load texture : ', mtl.Texture
      app.addMaterial(mat)  
    tset.Material = mat

def memorysize():
  pid = os.getpid()
  for task in os.popen("tasklist /fi \"pid eq %d\" /nh" % pid ).readlines():
    columns = task.split()
    if len(columns):
      return eval(columns[4].replace("\xff",""))
  return 0

def loadmodel(filename):
  basename, extension = os.path.splitext(os.path.basename(filename))
  c = app.createComponent('OBJ_Import')
  objData, matData = loadOBJ(filename,c) 
  if not buildproperty.Value:
    buildmodel(objData,matData,c)
  # finalize loading by applying the unit scale
  rx = 0
  if yupproperty.Value:
    rx = 90
  sp = 1
  if unitproperty.Value == "meters":
    sp = 1000
  if suppressproperty.Value:
    scalematrix = vcMatrix.new()
    scalematrix.rotateRelX(rx)
    scalematrix.scaleRel(sp,sp,sp)
    fea = findFeature(c.RootFeature,"Geometry")
    if fea:
      fea.Geometry.transformContent(scalematrix)
  else:  
    transform = findFeature(c.RootFeature,"Scale")
    if transform:
      transform.Expression = "Rx(%f).Sx(%f).Sy(%f).Sz(%f)" % (rx,sp,sp,sp)
  c.rebuild()
  return c

def findFeature(feat,name):
  if feat.Name == name:
    return feat
  for c in feat.Children:
    if c.Name == name:
      return c
  for c in feat.Children:
    ff = findFeature(c,name)
    if ff:
      return ff
  return None

def buildmodel(objData,matData,c):
  if suppressproperty.Value:
    fea = findFeature(c.RootFeature,"Geometry")
    if not fea:
      fea = c.RootFeature.createFeature(VC_GEOMETRY, "Geometry")
      fea.ShowBackfaces = True
    for group_name in objData:
      addFacesToGeometry(fea.Geometry,objData[group_name].Faces,objData[group_name].MaterialName)
  else:  
    transform = findFeature(c.RootFeature,"Scale")
    if not transform:
      transform = c.RootFeature.createFeature(VC_TRANSFORM, "Scale")
    for group_name in objData:
      fea = transform.createFeature(VC_GEOMETRY, group_name) 
      fea.ShowBackfaces = True
      addFacesToGeometry(fea.Geometry,objData[group_name].Faces,objData[group_name].MaterialName)

def first_state():
  global matData, filename, yupproperty, unitproperty, loopproperty, suppressproperty, buildproperty, progressproperty
  opencmd = app.findCommand("dialogOpen")
  opencmd.execute("",True,"OBJ FILE (*.obj)|*.obj")
  if not opencmd.Param_2:
    print "No file selected for importing, aborting command"
    return
  yupproperty = addProperty(getCommand(),VC_BOOLEAN,'YaxisUp',True,False,False,True,None,'')
  unitproperty = addStepProperty(getCommand(),VC_STRING,'Units',True,False,False,"meters",None,'',["millimeters","meters"])
  loopproperty = addProperty(getCommand(),VC_BOOLEAN,'AllFilesAndSubfolders',True,False,False,False,None,'')
  suppressproperty = addProperty(getCommand(),VC_BOOLEAN,'SuppressFeatures',True,False,False,False,None,'')
  buildproperty = addProperty(getCommand(),VC_BOOLEAN,'BuildWhileReading',False,False,False,True,None,'')
  progressproperty = addProperty(getCommand(),VC_BOOLEAN,'DisplayProgress',False,False,False,False,None,'')
  if displayPropertyDialog('ImportObj','Obj Import Parameters','') == False:
    print 'Command aborted.'
    return
  starttime = app.RealTime
  startMessage("Loading OBJ file(s)...")
  if loopproperty.Value:
    rootfolder = os.path.dirname(opencmd.Param_1[8:])
    for root, dirs, files in os.walk(rootfolder):
      for file in files:
        base, ext = os.path.splitext(file)
        if ext.lower() == ".obj":
          filename = os.path.join(root,file)
          vcmfilename = os.path.join(root,base) + ".vcm"
          if os.path.exists(vcmfilename):
            progressMessage("Skipping "+filename, 0.5)
          else:
            app.clearWorld()
            usedmemory = memorysize()
            if usedmemory > 1000000:
              endMessage("Memory full; please restart application and continue execution.")
              return
            try:
              progressMessage("Loading "+filename+"...",0.05)
              comp = loadmodel(filename)
              app.findCommand("fillView").execute()
              progressMessage("Saving "+vcmfilename+"...",0.7)
              comp.save("file:///"+vcmfilename)
            except:
              endMessage("Failed to process "+ filename+ ", restart application and try again.")
              return
    app.clearWorld()
  else:
    filename = opencmd.Param_1[8:]
    progressMessage("Loading "+filename+"...",0.05)
    comp = loadmodel(filename)
  app.findCommand("fillView").execute()
  app.render()
  endMessage("Completed loading in %.1f seconds" % (app.RealTime - starttime))

addState( first_state )